/*
 * Copyright 2006 Integsystem Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package test.jp.integ.struts.config.rule;

import jp.integ.struts.config.rule.QualifiedRule;
import junit.framework.TestCase;

/**
 * パッケージルートの処理ルールクラスのテストクラスです。
 * 
 * @author Jun Futagawa (Integsystem Corporation)
 * @version $Revision: 168 $ $Date: 2006-03-10 00:54:16 +0900 $
 */
public class PackageRootRuleTest extends TestCase {
	/**
	 * パッケージルートからルールに従いアクション定義パスを取得するテストです。
	 */
	public void testGetActionPath() {
		assertNull(QualifiedRule.getActionPath(null, null));
		assertEquals("/foo", QualifiedRule.getActionPath(null, "fooActionImpl"));
		assertEquals("/foo", QualifiedRule.getActionPath("", "fooActionImpl"));
		assertEquals("/foo/bar", QualifiedRule.getActionPath("",
				"foo.barActionImpl"));
		assertEquals("/bar", QualifiedRule.getActionPath("foo",
				"foo.barActionImpl"));
		assertEquals("/bar1/hoge", QualifiedRule.getActionPath("foo.*",
				"foo.bar1.hogeActionImpl"));
		assertEquals("/bar2/hoge", QualifiedRule.getActionPath("foo.*",
				"foo.bar2.hogeActionImpl"));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath("*.bar.*",
				"foo.bar.hoge.testActionImpl"));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.action", "foo.bar.hoge.action.testActionImpl"));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.testActionImpl"));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.action.testActionImpl"));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action|test)(action)",
				"foo.bar.hoge.test.testActionImpl"));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action|test)(action)",
				"foo.bar.hoge.action.testActionImpl"));
		assertEquals("/foo/hoge1/hoge2/test", QualifiedRule.getActionPath(
				"*.bar.*.*.(action)",
				"foo.bar.hoge1.hoge2.action.testActionImpl"));
		assertEquals("/foo/hoge/actionless/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.actionless.testActionImpl"));
		assertEquals("/foo/hoge/actionless/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.actionless.TestActionImpl"));
		assertEquals("/validate/validate", QualifiedRule.getActionPath(
				"org.seasar.struts.examples.*.(action)",
				"org.seasar.struts.examples.validate.ValidateAction"));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath("org.seasar.struts.examples.*.(action)",
								"org.seasar.struts.examples.employee.action.EmployeeSearchAction"));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action),org.seasar.struts.examples2.*.(action)",
								"org.seasar.struts.examples.employee.action.EmployeeSearchAction"));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action),org.seasar.struts.examples2.*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction"));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples2.*.(action),org.seasar.struts.examples.*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction"));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.(examples2|examples).*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction"));
	}

	public void testGetActionPathWithNoSplitUpperName() {
		assertNull(QualifiedRule.getActionPath(null, null, false));
		assertEquals("/foo", QualifiedRule.getActionPath(null, "fooActionImpl",
				false));
		assertEquals("/foo", QualifiedRule.getActionPath("", "fooActionImpl",
				false));
		assertEquals("/foo/bar", QualifiedRule.getActionPath("",
				"foo.barActionImpl", false));
		assertEquals("/bar", QualifiedRule.getActionPath("foo",
				"foo.barActionImpl", false));
		assertEquals("/bar1/hoge", QualifiedRule.getActionPath("foo.*",
				"foo.bar1.hogeActionImpl", false));
		assertEquals("/bar2/hoge", QualifiedRule.getActionPath("foo.*",
				"foo.bar2.hogeActionImpl", false));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath("*.bar.*",
				"foo.bar.hoge.testActionImpl", false));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.action", "foo.bar.hoge.action.testActionImpl", false));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.testActionImpl", false));
		assertEquals("/foo/hoge/test", QualifiedRule
				.getActionPath("*.bar.*.(action)",
						"foo.bar.hoge.action.testActionImpl", false));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action|test)(action)",
				"foo.bar.hoge.test.testActionImpl", false));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action|test)(action)",
				"foo.bar.hoge.action.testActionImpl", false));
		assertEquals("/foo/hoge1/hoge2/test", QualifiedRule.getActionPath(
				"*.bar.*.*.(action)",
				"foo.bar.hoge1.hoge2.action.testActionImpl", false));
		assertEquals("/foo/hoge/actionless/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.actionless.testActionImpl",
				false));
		assertEquals("/foo/hoge/actionless/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.actionless.TestActionImpl",
				false));
		assertEquals("/validate/validate", QualifiedRule.getActionPath(
				"org.seasar.struts.examples.*.(action)",
				"org.seasar.struts.examples.validate.ValidateAction", false));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action)",
								"org.seasar.struts.examples.employee.action.EmployeeSearchAction",
								false));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action),org.seasar.struts.examples2.*.(action)",
								"org.seasar.struts.examples.employee.action.EmployeeSearchAction",
								false));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action),org.seasar.struts.examples2.*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction",
								false));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples2.*.(action),org.seasar.struts.examples.*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction",
								false));
		assertEquals(
				"/employee/employeeSearch",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.(examples2|examples).*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction",
								false));
	}

	public void testGetActionPathWithSplitUpperName() {
		assertNull(QualifiedRule.getActionPath(null, null, true));
		assertEquals("/foo", QualifiedRule.getActionPath(null, "fooActionImpl",
				true));
		assertEquals("/foo", QualifiedRule.getActionPath("", "fooActionImpl",
				true));
		assertEquals("/foo/bar", QualifiedRule.getActionPath("",
				"foo.barActionImpl", true));
		assertEquals("/bar", QualifiedRule.getActionPath("foo",
				"foo.barActionImpl", true));
		assertEquals("/bar1/hoge", QualifiedRule.getActionPath("foo.*",
				"foo.bar1.hogeActionImpl", true));
		assertEquals("/bar2/hoge", QualifiedRule.getActionPath("foo.*",
				"foo.bar2.hogeActionImpl", true));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath("*.bar.*",
				"foo.bar.hoge.testActionImpl", true));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.action", "foo.bar.hoge.action.testActionImpl", true));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.testActionImpl", true));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.action.testActionImpl", true));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action|test)(action)",
				"foo.bar.hoge.test.testActionImpl", true));
		assertEquals("/foo/hoge/test", QualifiedRule.getActionPath(
				"*.bar.*.(action|test)(action)",
				"foo.bar.hoge.action.testActionImpl", true));
		assertEquals("/foo/hoge1/hoge2/test", QualifiedRule.getActionPath(
				"*.bar.*.*.(action)",
				"foo.bar.hoge1.hoge2.action.testActionImpl", true));
		assertEquals("/foo/hoge/actionless/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.actionless.testActionImpl",
				true));
		assertEquals("/foo/hoge/actionless/test", QualifiedRule.getActionPath(
				"*.bar.*.(action)", "foo.bar.hoge.actionless.TestActionImpl",
				true));
		assertEquals("/validate/validate", QualifiedRule.getActionPath(
				"org.seasar.struts.examples.*.(action)",
				"org.seasar.struts.examples.validate.ValidateAction", true));
		assertEquals(
				"/employee/employee/search",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action)",
								"org.seasar.struts.examples.employee.action.EmployeeSearchAction",
								true));
		assertEquals(
				"/employee/employee/search",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action),org.seasar.struts.examples2.*.(action)",
								"org.seasar.struts.examples.employee.action.EmployeeSearchAction",
								true));
		assertEquals(
				"/employee/employee/search",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples.*.(action),org.seasar.struts.examples2.*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction",
								true));
		assertEquals(
				"/employee/employee/search",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.examples2.*.(action),org.seasar.struts.examples.*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction",
								true));
		assertEquals(
				"/employee/employee/search",
				QualifiedRule
						.getActionPath(
								"org.seasar.struts.(examples2|examples).*.(action)",
								"org.seasar.struts.examples2.employee.action.EmployeeSearchAction",
								true));
	}

	public void testToUpperNameToPath() {
		assertNull(QualifiedRule.splitUpperName(null));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/foo/bar/hoge"));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/foo/BarHoge"));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/fooBarHoge"));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/FooBarHoge"));
		assertEquals("/foo/_/bar/hoge", QualifiedRule
				.splitUpperName("/Foo_BarHoge"));
		assertEquals("/foo/-/bar/hoge", QualifiedRule
				.splitUpperName("/Foo-BarHoge"));
	}
}
